/******************************************************************
 *
 * Name:	Plugin_procs.c
 * 
 * Description:
 * 
 * Author:	Tomasz Stachowiak
 *
 * Date:	18/07/96
 *
 * Change Log:
 *
 ********************************************************************/

#include <stdio.h>
#include <sys/types.h>

#include <dve/dve.h>

#include <Xm/Xm.h>

#include <sys/time.h>

#include "TSAudioPluginMotif.h"
#include "AudioPluginProtocol.h"
#include "gsm.h"


#define CODE_RATE 2

extern DveSession dveSession;       /* global variables */
int current_state;
XtAppContext app;
XtWorkProcId workproc;

gsm handle;

gsm_init()
  {
     handle = gsm_create();
  }

gsm_close()
  {
     gsm_destroy(handle);
  }


/**********************************************************************
 *
 * Functions kept for OpenDVE_procs.c compatibility
 *
 *********************************************************************/
void QueryPluginStatus(DveConfParticipant *part)
{
}

void ProcessClipboardData(char *format, char *data, int length)
{
}



/********************************************************************
 *
 * Frames management
 *
 *******************************************************************/
void DecodeAFrame(gsm_frame *frames)
  {
     char buffer[50/CODE_RATE][320];
     struct timeval start_time, stop_time;
     struct timeval rec_time;
     int i;
     int diff;
     
     gettimeofday(&start_time);
     for(i=0; i<50/CODE_RATE; i++)
       {
	  gsm_decode(handle, frames[i], buffer[i]);
       }
     gettimeofday(&stop_time);
     diff = (stop_time.tv_sec - start_time.tv_sec)*1000000 
	   + stop_time.tv_usec - start_time.tv_usec;
     diff = diff*CODE_RATE/50;
     printf("GSM decoding time in us: %d\n", diff);
     
     PutFrameToSpeaker(buffer, 160*50/CODE_RATE);
}


void DecodeVFrame(char *buf, int size)
  {
     static int first=1;

     if(first)
       {
	  decodeFrame(buf, size, 1);
	  first = 0;
       }
     else
	  decodeFrame(buf, size, 0);

  }


void
SendAFrame()
{
   audioPluginMessageStructure *msg;
   char sig[50/CODE_RATE][320];
   int size;
   struct timeval start_time, stop_time;
   int i;
   int diff;

   msg = (audioPluginMessageStructure *) dveMallocDataPacket(dveSession,
                                          sizeof(audioPluginMessageStructure));
   if (!msg)
   {
     printf("Message allocation error\n");
     return;
   }
 
   /* Get the audio from mic. */
   GetFrameFromMic(sig, 160*50/CODE_RATE);

   gettimeofday(&start_time);
   for(i=0; i < 50/CODE_RATE; i++)
     {
	gsm_encode(handle, sig[i], msg->frames[i]);
     }
   gettimeofday(&stop_time);
   diff = (stop_time.tv_sec - start_time.tv_sec)*1000000 + stop_time.tv_usec - 
          start_time.tv_usec;
   diff = diff*CODE_RATE/50;
   printf("GSM coding time in us: %d\n", diff);
   if(SendPluginMessage(dveSession,
                           AUDIO_PLUGIN_PORT, AUDIO, msg,
                           sizeof(audioPluginMessageStructure)) < 0)
		printf("Sending failure\n");

   return;
}

void
SendVFrame()
  {
     static int first=1;
     char buf[5120];
     int size;
     char *msg;

     if(first)
       {
	  size = codeIntraFrame(buf);
	  first = 0; 
       }
     else
	size = codeFrame(buf);
	  
     msg = (char *) dveMallocDataPacket(dveSession, size);
     memcpy(msg, buf, size);
     if(SendPluginMessage(dveSession, AUDIO_PLUGIN_PORT, VIDEO, msg, size) < 0)
	printf("Sending failure\n");
     
     return;
  }

void SendAVFrame()
  {
     SendAFrame();
     SendVFrame();

     if(current_state == START_STATE)
	workproc = XtAppAddWorkProc(app, SendAVFrame, NULL);
  }

/************************************************************************
 *
 * Conference mangement functions
 *
 ***********************************************************************/
void 
MemberLeft(DveConfParticipant member)
{
   fprintf(stderr, "User Left the Conference - %s@%s (%s)\n",
                    member.userName, member.hostName,
                    inet_ntoa(*(struct in_addr*)&((member.addr.v3))));
}

void 
MemberAdded(DveConfParticipant member)
{
   fprintf(stderr, "User Added to Conference - %s@%s (%s)\n",
                    member.userName, member.hostName,
                    inet_ntoa(*(struct in_addr*)&((member.addr.v3))));
}


/**************************************************************************
 *
 * Function processing incoming messages
 *
 *************************************************************************/
void 
ProcessPluginMessage(u_int32 messageType, void *data, int data_size, 
		     DveUserHandle sender) 
{
   audioPluginMessageStructure message;
   char buf[5120];

   switch (messageType){
      case AUDIO:
         memcpy(&message, data, sizeof(audioPluginMessageStructure));
         DecodeAFrame(message.frames);
	 break;

      case VIDEO:
         memcpy(buf, data, data_size);
	 DecodeVFrame(buf, data_size);
	 break;
   }
}


/*********************************************************************
 *
 * Plugin management functions
 *
 ********************************************************************/
void 
ShowPlugin(void)
{
   fprintf(stderr, "ShowPlugin\n");
}

void 
HidePlugin(void)
{
   fprintf(stderr, "HidePlugin\n");
}

void 
CleanupPlugin(void)
{
  CloseSpeaker();
}


/************************** END Plugin_procs.c **************************/






